#!/usr/bin/perl

#-----------------------------------------------------------
# migratel10nhelp.pl
#-----------------------------------------------------------
# A simple Perl script to help migrate existing localized 
# OOo help files from the 1.x to the 2.0 format
#-----------------------------------------------------------
#
# What you need:
# --------------
# Perl with XML::LibXML, XML::LibXSLT
# The transformation style sheet migrate.xsl
# The English help sources in 1.x format
# The English help sources in the initial 2.0 format
# The localized help sources in 2.0 format
#
# How to proceed:
# ---------------
# 1. Copy the localized help file tree in 1.x format
#    into the 1.x directory
# 2. Delete the following obsolete directories:
#    - handbuch
#    - setupguide
#    - screen
# 3. Rename all "common" directories to "shared"
# 4. Start the script
# 5. Check the output files in the 2.0 directory
#
# What you will get:
# ------------------
# A directory with the localized help files in 2.0 format.
# But please note:
# The files only contain elements that could be 
# migrated and that need localization. So you will end
# up with _incomplete_ help files. You will need to merge
# those file with the English 2.0 files. After merging
# you should have mixed language help files containing
# English and localized content as well as the structural
# elements that do not need localization 
#
# This script is only meant to simplify the business of
# migration.
#
# Use at your own risk.
# Check the resulting files.
# Feel free to tamper with this.
# Don't ask for support.

use File::Find;
use File::Basename;
use XML::LibXSLT;
use XML::LibXML;

$| = 1;

$lang = $ARGV[0] || &terminate("No language directory specified.");
$iso = $ARGV[1] || &terminate("No language ISO code specified.");

($root = $ARGV[2] || `pwd`) =~ s/\n//g;
$en_root = "$root/1.x/01/text";
$en2_root = "$root/2.0/01/text";
$l10n_root = "$root/1.x/$lang/text";
$l10n2_root = "$root/2.0/$lang/text";

$xslfile = "$root/migrate.xsl";
$xsl = &readxsl($xslfile);
$topids = "$root/alltopicids.txt";
$images = "$root/images.txt";

if (open TOP, $topids) {
	print "Reading $topids\n";
	$/ = "\n";
	@top = <TOP>;
    for (@top) {
    	chomp;
		($fn_,$id_) = split ";";
		$fn{$id_}=$fn_;
	}
	close TOP;
} else {
	&terminate("Cannot open $topids");
}

if (open IMG, $images) {
	print "Reading $images\n";
	$/ = "\n";
	@img = <IMG>;
    for (@img) {
    	chomp;
		($img_,$id_) = split ";";
		$img{$id_}=$img_;
	}
	close IMG;
} else {
	&terminate("Cannot open $images");
}

find(\&conv,$l10n_root);

sub conv {
	if (/\.xml$/) {
		($filepath =  $File::Find::name) =~ s/^$l10n_root//;
		$l10n_file = $l10n_root.$filepath;
		($l10n2_file = $l10n2_root.$filepath) =~ s/xml$/xhp/gs;;
		$en_file = $en_root.$filepath;
		$en2_file = $en2_root.$filepath;

		if (open L10N, $l10n_file) {
			if (open EN, $en_file) {
				if (open EN2, $en2_file) {
					undef $/;
					$l10n = <L10N>; close L10N;
					$en = <EN>; close EN;
					$en2 = <EN2>; close EN2;
					
					($topicid = $en2) =~ s/^.*topic[^>]+id="([^"]+)".*$/$1/gs;
					
					$xml = &xmltrans($l10n, $xsl);
					$xml_en = &xmltrans($en, $xsl);
				
					
					@l10n = split /<paragraph/, $xml;
					@en = split /<paragraph/, $xml_en;
					
					$nen = scalar @en;
					$nl10n = scalar @l10n;
					
					
								
					# both language arrays should be same size
					if ($nl10n != $nen) {
						print "ERROR Files do not match for $filepath (en=$nen $lang=$nl10n)\n";
					} else {
						for ($i=1; $i<=$#en; $i++) {
							($refno = $en[$i]) =~ s/^.*oldref="([^\"]+)".*$/$1/gis;
							#print "$refno\n";
							$l10n[$i] =~ s/oldref=\"([\"]*)\"/oldref="$refno"/gis;
							$l10n[$i] = "<paragraph".$l10n[$i];
							
							# Now check if the number of inline elements is consistent:
							# inline elements that must be consistent are:
							# image, embedvar, link, switchinline, variable, ahelp, caseinline, defaultinline
							
							$inline_en = scalar split /(<(image|embedvar|link|switchinline|variable|ahelp|caseinline|defaultinline))/, $en[$i];
							$inline_l10n = scalar split /(<(image|embedvar|link|switchinline|variable|ahelp|caseinline|defaultinline))/, $l10n[$i];
							if ($inline_en != $inline_l10n) { 
							  print "ERROR Inline Elements do not match for $filepath Ref $refno (en=$inline_en $lang=$inline_l10n)\n";
							}
						}
						
						$xml = join "",@l10n;


						
						# now synchronize the ids 
						$xml =~ s/(<paragraph[^>]+oldref="[^"])/§§§$1/gs;
						@xml_l10n = split /§§§/, $xml;
						$en2 =~ s/\n//gs;
						$en2 =~ s/(<paragraph[^>]+oldref="[^"])/§§§$1/gs;
						@xml_en2 = split /§§§/, $en2;
						
						for $e(@xml_en2) {
							($id = $e) =~ s/^[^>]+id="([^"]*)".*$/$1/gs;
							($old = $e) =~ s/^[^>]+oldref="([^"]*)".*$/$1/gs;
							$id{$old} = $id;
						}
						
						if ($#xml_l10n != $#xml_en2) {
							$nen = scalar @xml_en2;
							$nl10n = scalar @xml_l10n;
							print "error Files do not match for $filepath continuing anyway (new_en=$nen $lang=$nl10n)\n";
						} 
						for ($i=1; $i<=$#xml_l10n; $i++) { 
						  $xml_l10n[$i] =~ s/(^<[^>]+)(oldref=")([^"]+)(".*$)/${1}id="$id{$3}" $2$3$4/gs;
						}
						
						$xml = join "", @xml_l10n;
				
						&savefile;
					}
				} else {
					print "ERROR No new English file for $filepath\n";
				}
			} else {
				print "ERROR No English file for $filepath\n";
			}
		} else {
			print "ERROR Cannot open $l10n_file\n";
		}
	}
}

sub savefile { 
	$out = $l10n2_file;
	&makedir($out) if (!-e dirname($out));
	
	if (open OUT, ">$out") {
		print OUT $xml;
		close OUT;
		print "DONE  $filepath done\n";
	} else {
		print "ERROR Cannot write to $l10n_file\n";
	}
}

sub makedir {
	my @path;
	$targetdir = dirname(shift);
	return (-1) if (!($targetdir =~ /^\/.*$/));
	while (!($targetdir =~ /^\/$/))	{
		push @path, basename($targetdir);
		$targetdir = dirname($targetdir);
	}
	for $p (reverse(@path)) {
		$targetdir = $targetdir."/".$p;
		mkdir $targetdir if (!-e ($targetdir));
	}
}

sub xmltrans { # makes the xslt tranformation
	my $xml = shift;
	my $xsl = shift;                              # The file name of the xsl file
	
	my $parser = XML::LibXML->new();
	my $xslt = XML::LibXSLT->new();

	my $source = $parser->parse_string($xml);
	my $style_doc = $parser->parse_string($xsl);

	my $stylesheet = $xslt->parse_stylesheet($style_doc);
	
	XML::LibXSLT->register_function("urn:ooohelp", "resid", \&returnfnid);
	XML::LibXSLT->register_function("urn:ooohelp", "resimgid", \&returnimgid);
	
	my $result = $stylesheet->transform($source,	XML::LibXSLT::xpath_to_string(iso => "$iso"));

	#return XML::LibXML::encodeToUTF8("utf-8",$stylesheet->output_string($result));
	return $stylesheet->output_string($result);
}

sub readxsl {
	my $xslfile = shift;
	print "Reading XSLT $xslfile\n";
	if (open XSL, $xslfile) {
		undef $/;
		my $xsl = <XSL>;
		close XSL;
		return $xsl;
	} else {
		die "Fatal: Error opening $xslfile: $!";
	}
}

sub terminate {
	my $msg = shift;
	$out = <<"OUT";
$msg
	
migratel10nhelp.pl languagedir ISO [rootdir]
	language = Subdirectory with language files
	ISO      = complete language ISO code (eg. fr-FR for French)
	rootdir  = Path to this script and the subdirectories, default = current dir
OUT
	print $out;
	die "\n";
}

sub returnfnid {
	#print "FN: $_[0]\n";
	if (defined $fn{$_[0]}) {
		return $fn{$_[0]};
	} else {
		return "NOTFOUND_$_[0]";
	} 
}

sub returnimgid { 
	#print "IMG: $id\n";
	if (defined $img{$_[0]}) {
		return $img{$_[0]};
	} else {
		return "NOTFOUND_$_[0]";
	} 
}
